=begin
Copyright 2018 (c) TIG
All Rights Reserved.
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED 
WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
Usage:
Preselect geometry [containing some edges] and/or containers [group/component-instance].
Menu: Edit > Remove Edge Material
Or type in Ruby Console TIG::RemoveEdgeMaterial.now
=end

module TIG

  module RemoveEdgeMaterial
  
    # menu set up
    unless @menu
      UI.menu("Edit").add_item("Remove Edge Material"){ self.now() }
    end
    @menu = true
    #
  
    def self.now()
      @count = 0
      @hcount = 0
      @hidden = true
      @asked = false
      model = Sketchup.active_model
      @ss = model.selection
       model.start_operation("Remove Edge Material", true) # one step undo
      ss=@ss.to_a
      @ss.clear
      if es = ss.grep(Sketchup::Edge) # process selected edges
        self.process_edges(es)
      end
      if gs = ss.grep(Sketchup::Group)
        self.process_groups(gs)
      end
      if cs = ss.grep(Sketchup::ComponentInstance)
        self.process_compos(cs)
      end
      @ss.clear
      unless es[0] || gs[0] || cs[0]
        UI.messagebox("Remove Edge Material:\n\nMust Preselect Geometry with Edges and/or Groups/Components...")
        model.abort_operation
      else
        msg = "Edges Material Removed."
        msg << "\nIncluding #{@hcount} Hidden/Smoothed Edges." if @hcount > 0
        UI.messagebox("#{@count} #{msg}")
        model.commit_operation
      end
    end
    
    def self.get_hidden(es)
      faces = []
      es.each{|e| faces << e.faces }
      faces.flatten!
      faces.uniq!
      hs = []
      faces.each{|f|
        f.edges.each{|e|
          next if ! e.material || es.include?(e)
          hs << e if e.hidden? || e.smooth?
        }
      }
      hs.flatten!
      hs.uniq!
      return hs
    end
    
    def self.get_face_hidden(es)
      faces = @ss.grep(Sketchup::Face)
      fs = []
      faces.each{|f|
        f.edges.each{|e|
          next if ! e.material || es.include?(e)
          fs << e if e.hidden? || e.smooth?
        }
      }
      fs.flatten!
      fs.uniq!
      return fs
    end
    
    def self.process_edges(es=[]) # default passed edges
      if @hidden
        hs = self.get_hidden(es)
        fs = self.get_face_hidden(es)
        hs = hs + fs
        hs.uniq!
        if hs[0]
          unless @asked # only ask once
            UI.beep
            @ss.add(hs)
            yn = UI.messagebox("Remove Edge Material:\n\nEdges Or Surfaces in The Preselection Have Some Faces\nWith #{hs.length} Unselected Hidden/Smoothed Edges\nDo You Also Want To Process Those ?", MB_YESNO) # 6=YES 7=NO
            @asked = true # only ask once
            if yn == 6
              @hidden = true
            else
              @hidden = false
            end
          end
          if @hidden
            es = es + hs
            @hcount+= hs.length
          end
        end
      end
      # process edges
      es.each{|e|
        if e.material
          e.material = nil
          @count+=1
        end
      }
    end
    
    def self.process_groups(gs=[]) # nested in groups
      gs.each{|g|
        es = g.entities.grep(Sketchup::Edge)
        self.process_edges(es) if es[0]
        gg = g.entities.grep(Sketchup::Group)
        self.process_groups(gg) if gg[0]
        cc = g.entities.grep(Sketchup::ComponentInstance)
        self.process_compos(cc) if cc[0]
      }
    end
    
    def self.process_compos(cs=[]) # nested in definitions
      cs.each{|c|
        es = c.definition.entities.grep(Sketchup::Edge)
        self.process_edges(es) if es[0]
        gg = c.definition.entities.grep(Sketchup::Group)
        self.process_groups(gg) if gg[0]
        cc = c.definition.entities.grep(Sketchup::ComponentInstance)
        self.process_compos(cc) if cc[0]
      }
    end
    
  end
  
end
